#!/bin/bash

degree=$1
prompt=$2

shopt -s extglob

if [[ -z $degree || $degree != +([0-9]) ]]
then
    echo "error: degree is not specified or is not a digit string"
    exit 1
fi

if [[ -z $prompt ]]
then
    echo "error: prompt is not specified"
    exit 1
fi

echo \
"degree = ${degree}

define prompt() {
  if (interactive) print \"${prompt}: \"
}

define display_e(i) {
  print \"[\",x0[i]
$(for ((i=1; i<${degree}; ++i))
do echo "  print \",\",x${i}[i]"
done)
  print \"]\"
  if (denominator[i] == 1) print \"\n\" else print \"/\", denominator[i], \"\n\"
}

define reduce_e(o) {
  auto t
  t = denominator[o]
$(for ((i=0; i<${degree}; ++i))
do echo "  t = gcd(x${i}[o],t)"
done)
  if (denominator[o] < 0) t = - t;
$(for ((i=0; i<${degree}; ++i))
do echo "  x${i}[o] /= t"
done)
  denominator[o] /= t
}

define extract_beb(o,i1,i2) {
  auto j
  j = numerator[i2]
  if (j < 1 || degree < j || denominator[i2] != 1) {
    ignore = error_message(7)
    return
  }
$(for ((i=0; i<${degree}; ++i))
do echo "  if (j == $((i+1))) numerator[o] = x${i}[i1]"
done)
  denominator[o] = denominator[i1]
  ignore = reduce_b(o)
}

define convert_eb(o,i) {
  x0[o] = numerator[i]
$(for ((i=1; i<${degree}; ++i))
do echo "  x${i}[o] = 0"
done)
  denominator[o] = denominator[i]
}

define reset_e(o) {
$(for ((i=0; i<${degree}; ++i))
do echo "  x${i}[o] = 0"
done)
  denominator[o] = 1
}

define initialize_ebl(o,i1,i2) {
$(for ((i=0; i<${degree}; ++i))
do echo "  x${i}[o] *= denominator[i1]"
done)
$(for ((i=0; i<${degree}; ++i))
do echo "  if (i2 == $((i+1))) x${i}[o] = numerator[i1]*denominator[o]"
done)
  denominator[o] *= denominator[i1]
  ignore = reduce_e(o)
}

define copy_ee(o,i) {
$(for ((i=0; i<${degree}; ++i))
do echo "  x${i}[o] = x${i}[i]"
done)
  denominator[o] = denominator[i]
}

define negate_ee(o,i) {
$(for ((i=0; i<${degree}; ++i))
do echo "  x${i}[o] = - x${i}[i]"
done)
  denominator[o] = denominator[i]
}

define add_eee(o,i1,i2) {
$(for ((i=0; i<${degree}; ++i))
do echo "  x${i}[o] = x${i}[i1]*denominator[i2] + x${i}[i2]*denominator[i1]"
done)
  denominator[o] = denominator[i1] * denominator[i2]
  ignore = reduce_e(o)
}

define subtract_eee(o,i1,i2) {
$(for ((i=0; i<${degree}; ++i))
do echo "  x${i}[o] = x${i}[i1]*denominator[i2] - x${i}[i2]*denominator[i1]"
done)
  denominator[o] = denominator[i1] * denominator[i2]
  ignore = reduce_e(o)
}

define multiply_eeb(o,i1,i2) {
$(for ((i=0; i<${degree}; ++i))
do echo "  x${i}[o] = x${i}[i1] * numerator[i2]"
done)
  denominator[o] = denominator[i1] * denominator[i2]
  ignore = reduce_e(o)
}

define multiply_eee(o,i1,i2) {
$(s=0
for ((i=0; i<${degree}; ++i))
do plus_sign=' '
   for ((j=0; j<${degree}; ++j))
   do for ((k=0; k<${degree}; ++k))
      do echo "  x${i}[o] ${plus_sign}= mult[${s}]*x${j}[i1]*x${k}[i2]"
         s=$((s+1))
         plus_sign=+
      done
   done
done)
  denominator[o] = denominator[i1] * denominator[i2]
  ignore = reduce_e(o)
}

define conjugate_eeb(o,i1,i2) {
  auto offset,mtrans
  if (denominator[i2] != 1) {
    ignore = error_message(8)
    return
  }
  if (trans_grp_init[numerator[i2]] != 1) {
    ignore = error_message(22)
    return
  }
  mtrans = trans_grp_fix[numerator[i2]]
  if (mtrans < 0 || degree <= mtrans) {
    ignore = error_message(23)
    return
  }
  offset = mtrans * degree^2
$(s=0
for ((i=0; i<${degree}; ++i))
do plus_sign=' '
   for ((j=0; j<${degree}; ++j))
   do echo "  x${i}[o] ${plus_sign}= conj[offset+${s}]*x${j}[i1]"
      s=$((s+1))
      plus_sign=+
   done
done)
  denominator[o] = denominator[i1]
  ignore = reduce_e(o)
}

define locate(i) {
  auto error,count,loc
  if (denominator[i] != 1) {
    ignore = error_message(13)
    return -1
  }
  error = 0
$(for ((i=0; i<${degree}; ++i))
do echo "  if (x${i}[i]!=0 && x${i}[i]!=1) error=1"
done)
  if (error == 1) {
    print \"i=\", i, \"\n\"
    ignore = error_message(14)
    ignore = display_e(i)
    return -1
  }
  count = 0
$(for ((i=0; i<${degree}; ++i))
do echo "  if (x${i}[i]==1) count=count+1"
done)
  if (count != 1) {
    ignore = error_message(15)
    return -1
  }
  loc = 0
$(for ((i=0; i<${degree}; ++i))
do echo "  if (x${i}[i]==1) loc=${i}"
done)
  return (loc)
}

define initialize_mult_eee(l_arg,r_arg,res) {
  auto n,m,offset1,offset2
  n = locate(l_arg)
  if (n == -1) return
  m = locate(r_arg)
  if (m == -1) return
  offset1 = n*degree + m;
  offset2 = m*degree + n;
$(for ((i=0; i<${degree}; ++i))
do echo "  mult[offset1] = x${i}[res];"
   echo "  offset1 += degree^2;"
   echo "  mult[offset2] = x${i}[res];"
   echo "  offset2 += degree^2;"
done)
}

define initialize_conj_ebe(l_arg,r_arg,res) {
  auto n,m,mtrans,offset
  n = locate(l_arg)
  if (n == -1) {
    ignore = error_message(24)
    return
  }
  m = numerator[r_arg]
  if (denominator[r_arg] != 1) {
    ignore = error_message(25)
    return
  }
  mtrans = grp_trans(m)
  offset = mtrans*degree^2 + n;
$(for ((i=0; i<${degree}; ++i))
do echo "  conj[offset+$((i*degree))] = x${i}[res];"
done)
}

for (i=0; i<degree^3; ++i) {
  mult[i] = 0
  conj[i] = 0
}

define is_zero_e(i) {
$(for ((i=0; i<${degree}; ++i))
do echo "  if (x${i}[i] != 0) return 0"
done)
  return 1
}"
